@@ -51,13 +51,6 @@ backend (by default keyring guesses, usually correctly, for example
 you get KDE Wallet under KDE, and Gnome Keyring under Gnome or Unity).
 '''
 
-from mercurial import util, sslutil
-from mercurial.i18n import _
-from mercurial.url import passwordmgr
-from mercurial import mail
-from mercurial.mail import SMTPS, STARTTLS
-from mercurial import encoding
-
 import urllib2
 import smtplib
 import socket
@@ -65,6 +58,13 @@ import os
 import sys
 import re
 
+from mercurial import util, sslutil
+from mercurial.i18n import _
+from mercurial.url import passwordmgr
+from mercurial import mail
+from mercurial.mail import SMTPS, STARTTLS
+from mercurial import encoding
+
 # pylint: disable=invalid-name, line-too-long, protected-access, too-many-arguments
 
 ###########################################################################
@@ -140,7 +140,6 @@ def import_keyring():
         meu.disable_logging("keyring")
     return mod
 
-
 #################################################################
 # Actual implementation
 #################################################################
@@ -254,6 +253,14 @@ _re_http_url = re.compile(r'^https?://')
 def is_http_path(url):
     return bool(_re_http_url.search(url))
 
+
+def make_passwordmgr(ui):
+    """Constructing passwordmgr in a way compatible with various mercurials"""
+    if hasattr(ui, 'httppasswordmgrdb'):
+        return passwordmgr(ui, ui.httppasswordmgrdb)
+    else:
+        return passwordmgr(ui)
+
 ############################################################
 # HTTP password management
 ############################################################
@@ -304,9 +311,13 @@ class HTTPPasswordHandler(object):
             return url_user, url_passwd, self.SRC_URL, base_url
 
         # Extract data from urllib (in case it was already stored)
-        urllib_user, urllib_pwd \
-            = urllib2.HTTPPasswordMgrWithDefaultRealm.find_user_password(
-                pwmgr, realm, authuri)
+        if isinstance(pwmgr, urllib2.HTTPPasswordMgrWithDefaultRealm):
+            urllib_user, urllib_pwd = \
+                urllib2.HTTPPasswordMgrWithDefaultRealm.find_user_password(
+                    pwmgr, realm, authuri)
+        else:
+            urllib_user, urllib_pwd = pwmgr.passwddb.find_user_password(
+                realm, authuri)
         if urllib_user and urllib_pwd:
             return urllib_user, urllib_pwd, self.SRC_URLCACHE, base_url
 
@@ -389,7 +400,15 @@ class HTTPPasswordHandler(object):
             # Otherwise we won't be able to find the password so it
             # does not make much sense to preserve it
             _debug(ui, _("Saving password for %s to keyring") % user)
-            password_store.set_http_password(final_url, user, pwd)
+            try:
+                password_store.set_http_password(final_url, user, pwd)
+            except Exception, e:
+                keyring = import_keyring()
+                if isinstance(e, keyring.errors.PasswordSetError):
+                    ui.traceback()
+                    ui.warn(_("warning: failed to save password in keyring\n"))
+                else:
+                    raise e
 
         # Saving password to the memory cache
         self.pwd_cache.store(realm, final_url, user, pwd)
@@ -617,7 +636,9 @@ def keyring_supported_smtp(ui, username):
             raise util.Abort(_('invalid smtp.verifycert configuration: %s')
                              % (verifycert))
         verifycert = False
-    if (starttls or smtps) and verifycert:
+    if getattr(sslutil, 'sslkwargs', None) is None:
+        sslkwargs = None
+    elif (starttls or smtps) and verifycert:
         sslkwargs = sslutil.sslkwargs(ui, mailhost)
     else:
         sslkwargs = {}
@@ -625,12 +646,16 @@ def keyring_supported_smtp(ui, username):
         ui.note(_('(using smtps)\n'))
 
         # mercurial 3.8 added a mandatory host arg
-        if 'host' in SMTPS.__init__.__code__.co_varnames:
+        if not sslkwargs:
+            s = SMTPS(ui, local_hostname=local_hostname, host=mailhost)
+        elif 'host' in SMTPS.__init__.__code__.co_varnames:
             s = SMTPS(sslkwargs, local_hostname=local_hostname, host=mailhost)
         else:
             s = SMTPS(sslkwargs, local_hostname=local_hostname)
     elif starttls:
-        if 'host' in STARTTLS.__init__.__code__.co_varnames:
+        if not sslkwargs:
+            s = STARTTLS(ui, local_hostname=local_hostname, host=mailhost)
+        elif 'host' in STARTTLS.__init__.__code__.co_varnames:
             s = STARTTLS(sslkwargs, local_hostname=local_hostname, host=mailhost)
         else:
             s = STARTTLS(sslkwargs, local_hostname=local_hostname)
@@ -651,7 +676,10 @@ def keyring_supported_smtp(ui, username):
         s.ehlo()
     if (starttls or smtps) and verifycert:
         ui.note(_('(verifying remote certificate)\n'))
-        sslutil.validator(ui, mailhost)(s.sock, verifycert == 'strict')
+        if getattr(sslutil, 'validatesocket', None):
+            sslutil.validatesocket(s.sock)
+        else:
+            validator(ui, mailhost)(s.sock, verifycert == 'strict')
 
     # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     stored = password = password_store.get_smtp_password(
@@ -746,7 +774,7 @@ def cmd_keyring_check(ui, repo, *path_args, **opts):   # pylint: disable=unused-
                 ui.status(_("    %s: non-http path (%s)\n") % (name, url))
             continue
         user, pwd, source, final_url = handler.get_credentials(
-            passwordmgr(ui), name, url)
+            make_passwordmgr(ui), name, url)
         if pwd:
             ui.status(_("    %s: password available, source: %s, bound to user %s, url %s\n") % (
                 name, source, user, final_url))
@@ -781,7 +809,7 @@ def cmd_keyring_clear(ui, repo, path, **opts):  # pylint: disable=unused-argumen
     handler = HTTPPasswordHandler()
 
     user, pwd, source, final_url = handler.get_credentials(
-        passwordmgr(ui), path, path_url)
+        make_passwordmgr(ui), path, path_url)
     if not user:
         ui.status(_("Username not configured for url %s\n") % final_url)
         return
